/**
 * @file toggle.c
 * This program demonstrates starting a PASM COG
 * passing parameters from a C program to a PASM program.
 *
 * C to PASM Mailbox example
 *
 * WARNING: This code makes all IO pins except 30/31 toggle HIGH/LOW. Check if this is OK 
 * for the board you are using.
 *
 *
 * to use:
 * from directory containing source
 * make clean
 * make
 * propeller-load -pn -t -r toggle.elf  (where n is port #)
 *
 * Copyright (c) 2011, Steve Denson, Rick Post
 * MIT Licensed - terms of use below.
 */

#include <stdio.h>
#include <propeller.h>					// propeller specific definitions

// the mailbox structure (STRUCT) for communication to PASM routine 
//
typedef struct 
{
unsigned int pins; // a pointer to this gets passed to the PASM code as the PAR register
unsigned int wait_time;
} mbox_type;

// the mailbox is defined as VOLATILE since these variable are being used to communicate with an execution thread
// outside the scope of this C program (an independent COG running PASM in this case. They are read/write variable
// whos values could be changed outside the control of this program. VOLATILE also prevent the gcc compiler from 
// optimizing away these variables if it sees cases where it might optimize and unchanging variable. This will be 
// made clear in a future example.

static volatile mbox_type mb;

// this constant is used in the whiel loop below to controll the highest flash rate
#define MIN_GAP 400000

// C stub function to start the PASM routine
// You provide the entry point to the PASM
// and a pointer to the STATIC HUB mailbox
// The cognew function in the propeller.c library returns the COG #
//
int start( volatile unsigned int *pinptr)
{
    extern unsigned int binary_toggle_dat_start;
    cognew(&binary_toggle_dat_start, (void *)pinptr);
}

// C main function
// LMM model
void main (int argc,  char* argv[])
{
    printf("hello, world!\n");							// let the lead LMM COG say hello
    mb.wait_time = CLKFREQ;								// set the delay rate in the STATIC mailbox
    													// this is actually the duty cycle of the blink 0.5 sec on, 0.5 sec off
    mb.pins = 0x3fFFffff; 								// set the PIN mask into the STATIC mailbox
    													// light up all pins except 30 & 31 since we don't know board config
    													// The PASM code is written to read these variables into local PASM 
    													// variable when it initially start up.
    printf ("New COG# %d started.\n",start(&mb.pins));  // start a new COG passing a pointer to the STATIC mailbox structure
    
    while(1) {											// start infinite loop to toggel pins at a variable rate
      sleep(2);											// change the flash rate every 2 seconds
      mb.wait_time =  mb.wait_time >> 1;				// divide the current wait time by 2 (shift right 1 bit)
      													// this changes the C variable in the mailbox
      													// the PASM code will only notice the change if it does a RDLONG of the 
      													// variable from time to time - the PASM code provide does this through each
      													// toggle loop
      if (mb.wait_time < MIN_GAP)						// don't let it get too fast - can't see the blinks anymore
	     mb.wait_time = _clkfreq;						// start back at a 1 second rate if we got too fast
    }   
}

/*
    +--------------------------------------------------------------------
      TERMS OF USE: MIT License
    +--------------------------------------------------------------------
    Permission is hereby granted, free of charge, to any person obtaining
    a copy of this software and associated documentation files
    (the "Software"), to deal in the Software without restriction,
    including without limitation the rights to use, copy, modify, merge,
    publish, distribute, sublicense, and/or sell copies of the Software,
    and to permit persons to whom the Software is furnished to do so,
    subject to the following conditions:

    The above copyright notice and this permission notice shall be
    included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
    +--------------------------------------------------------------------
*/ 